"""Add EMS objects only for MultipleZone."""


def addGlobVarListMultipleZone(self, verboseMode: bool = True):
    """Remove existing Global Variable objects and add correct Global Variable objects for MultipleZone accim."""
    globalvariablelist = ([program for program in self.idf1.idfobjects['ENERGYMANAGEMENTSYSTEM:GLOBALVARIABLE']])

    for i in range(len(globalvariablelist)):
        firstglobalvariablelist = self.idf1.idfobjects['ENERGYMANAGEMENTSYSTEM:GLOBALVARIABLE'][-1]
        self.idf1.removeidfobject(firstglobalvariablelist)

    del globalvariablelist

    self.idf1.newidfobject(
        'EnergyManagementSystem:GlobalVariable',
        Erl_Variable_1_Name='ACST',
        Erl_Variable_2_Name='AHST',
        Erl_Variable_3_Name='ACSTnoTol',
        Erl_Variable_4_Name='AHSTnoTol',
        Erl_Variable_5_Name='AdapStand',
        Erl_Variable_6_Name='ACSTaul',
        Erl_Variable_7_Name='ACSTall',
        Erl_Variable_8_Name='AHSTaul',
        Erl_Variable_9_Name='AHSTall',
        Erl_Variable_10_Name='CAT',
        Erl_Variable_11_Name='ACSToffset',
        Erl_Variable_12_Name='AHSToffset',
        Erl_Variable_13_Name='ComfMod',
        Erl_Variable_14_Name='ComfTemp',
        Erl_Variable_15_Name='ACSTtol',
        Erl_Variable_16_Name='AHSTtol',
        Erl_Variable_17_Name='VST',
        Erl_Variable_18_Name='VSToffset',
        Erl_Variable_19_Name='MaxWindSpeed',
        Erl_Variable_20_Name='VentCtrl',
        Erl_Variable_21_Name='HVACmode',
        Erl_Variable_22_Name='MinOutTemp',
        Erl_Variable_23_Name='MinOToffset'
        )

    for zonename in self.zonenames:
        self.idf1.newidfobject(
            'EnergyManagementSystem:GlobalVariable',
            Erl_Variable_1_Name='ComfHours_'+zonename,
            Erl_Variable_2_Name='DiscomfAppHotHours_'+zonename,
            Erl_Variable_3_Name='DiscomfAppColdHours_'+zonename,
            Erl_Variable_4_Name='DiscomfNonAppHotHours_'+zonename,
            Erl_Variable_5_Name='DiscomfNonAppColdHours_'+zonename,
            Erl_Variable_6_Name='ComfHoursNoApp_'+zonename
            )
    if verboseMode:
        print("Global variables objects have been added")



def addEMSSensorsMultipleZone(self, verboseMode: bool = True):
    """Add EMS sensors for MultipleZone accim."""
    sensorlist = ([sensor.Name for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor']])

    if 'RMOT' in sensorlist:
        if verboseMode:
            print('Not added - RMOT Sensor')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Sensor',
            Name='RMOT',
            OutputVariable_or_OutputMeter_Index_Key_Name='People '+self.zonenames_orig[0],
            OutputVariable_or_OutputMeter_Name='Zone Thermal Comfort CEN 15251 Adaptive Model Running Average Outdoor Air Temperature'
            )
        if verboseMode:
            print('Added - RMOT Sensor')
    #    print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name=='RMOT'])

    if 'PMOT' in sensorlist:
        if verboseMode:
            print('Not added - PMOT Sensor')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Sensor',
            Name='PMOT',
            OutputVariable_or_OutputMeter_Index_Key_Name='People '+self.zonenames_orig[0],
            OutputVariable_or_OutputMeter_Name='Zone Thermal Comfort ASHRAE 55 Adaptive Model Running Average Outdoor Air Temperature'
            )
        if verboseMode:
            print('Added - PMOT Sensor')
    #    print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name=='PMOT'])

    for i in range(len(self.zonenames)):
        if self.zonenames[i]+'_OpT' in sensorlist:
            if verboseMode:
                print('Not added - '+self.zonenames[i]+'_OpT Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.zonenames[i]+'_OpT',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.zonenames_orig[i],
                OutputVariable_or_OutputMeter_Name='Zone Operative Temperature'
                )
            if verboseMode:
                print('Added - '+self.zonenames[i]+'_OpT Sensor')
    #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.zonenames[i]+'_OpT'])
        if self.zonenames[i]+'_CoolCoil' in sensorlist:
            if verboseMode:
                print('Not added - '+self.zonenames[i]+'_CoolCoil Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.zonenames[i]+'_CoolCoil',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.zonenames_orig[i]+' VRF Indoor Unit DX Cooling Coil',
                OutputVariable_or_OutputMeter_Name='Cooling Coil Total Cooling Rate'
                )
            if verboseMode:
                print('Added - '+self.zonenames[i]+'_CoolCoil Sensor')
    #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.zonenames[i]+'_CoolCoil'])
        if self.zonenames[i]+'_HeatCoil' in sensorlist:
            if verboseMode:
                print('Not added - '+self.zonenames[i]+'_HeatCoil Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.zonenames[i]+'_HeatCoil',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.zonenames_orig[i]+' VRF Indoor Unit DX Heating Coil',
                OutputVariable_or_OutputMeter_Name='Heating Coil Heating Rate'
                )
            if verboseMode:
                print('Added - '+self.zonenames[i]+'_HeatCoil Sensor')
    #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.zonenames[i]+'_HeatCoil'])
        if self.zonenames[i]+'_WindSpeed' in sensorlist:
            if verboseMode:
                print('Not added - '+self.zonenames[i]+'_WindSpeed Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.zonenames[i]+'_WindSpeed',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.zonenames_orig[i],
                OutputVariable_or_OutputMeter_Name='Zone Outdoor Air Wind Speed'
                )
            if verboseMode:
                print('Added - '+self.zonenames[i]+'_WindSpeed Sensor')
    #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.zonenames[i]+'_WindSpeed'])
        if self.zonenames[i]+'_OutT' in sensorlist:
            if verboseMode:
                print('Not added - '+self.zonenames[i]+'_OutT Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.zonenames[i]+'_OutT',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.zonenames_orig[i],
                OutputVariable_or_OutputMeter_Name='Zone Outdoor Air Drybulb Temperature'
                )
            if verboseMode:
                print('Added - '+self.zonenames[i]+'_OutT Sensor')
    #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.zonenames[i]+'_OutT'])

    # if 'HVACConsump' in sensorlist:
    #     print('Not added - HVACConsump Sensor')
    # else:
    #     self.idf1.newidfobject(
    #         'EnergyManagementSystem:Sensor',
    #         Name='HVACConsump',
    #         OutputVariable_or_OutputMeter_Index_Key_Name='Whole Building',
    #         OutputVariable_or_OutputMeter_Name='Facility Total HVAC Electric Demand Power'
    #         )
    #     print('Added - HVACConsump Sensor')
    #     print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name=='HVACConsump'])

    for i in range(len(self.windownamelist)):
        if self.windownamelist[i]+'_OpT' in sensorlist:
            if verboseMode:
                print('Not added - '+self.windownamelist[i]+'_OpT Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.windownamelist[i]+'_OpT',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.windownamelist_orig_split[i][0],
                OutputVariable_or_OutputMeter_Name='Zone Operative Temperature'
            )
            if verboseMode:
                print('Added - '+self.windownamelist[i]+'_OpT Sensor')

        if self.windownamelist[i]+'_CoolCoil' in sensorlist:
            if verboseMode:
                print('Not added - '+self.windownamelist[i]+'_CoolCoil Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.windownamelist[i]+'_CoolCoil',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.windownamelist_orig_split[i][0]+' VRF Indoor Unit DX Cooling Coil',
                OutputVariable_or_OutputMeter_Name='Cooling Coil Total Cooling Rate'
                )
            if verboseMode:
                print('Added - '+self.windownamelist[i]+'_CoolCoil Sensor')

        if self.windownamelist[i]+'_HeatCoil' in sensorlist:
            if verboseMode:
                print('Not added - '+self.windownamelist[i]+'_HeatCoil Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.windownamelist[i]+'_HeatCoil',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.windownamelist_orig_split[i][0]+' VRF Indoor Unit DX Heating Coil',
                OutputVariable_or_OutputMeter_Name='Heating Coil Heating Rate'
                )
            if verboseMode:
                print('Added - '+self.windownamelist[i]+'_HeatCoil Sensor')

        if self.windownamelist[i]+'_WindSpeed' in sensorlist:
            if verboseMode:
                print('Not added - '+self.windownamelist[i]+'_WindSpeed Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.windownamelist[i]+'_WindSpeed',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.windownamelist_orig_split[i][0],
                OutputVariable_or_OutputMeter_Name='Zone Outdoor Air Wind Speed'
                )
            if verboseMode:
                print('Added - '+self.windownamelist[i]+'_WindSpeed Sensor')

        if self.windownamelist[i]+'_OutT' in sensorlist:
            if verboseMode:
                print('Not added - '+self.windownamelist[i]+'_WindSpeed Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.windownamelist[i]+'_OutT',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.windownamelist_orig_split[i][0],
                OutputVariable_or_OutputMeter_Name='Zone Outdoor Air Drybulb Temperature'
                )
            if verboseMode:
                print('Added - '+self.windownamelist[i]+'_OutT Sensor')

    if 'OutT' in sensorlist:
        if verboseMode:
            print('Not added - OutT Sensor')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Sensor',
            Name='OutT',
            OutputVariable_or_OutputMeter_Index_Key_Name='Environment',
            OutputVariable_or_OutputMeter_Name='Site Outdoor Air Drybulb Temperature'
            )
        if verboseMode:
            print('Added - OutT Sensor')

    del sensorlist


def addEMSActuatorsMultipleZone(self, verboseMode: bool = True):
    """Add EMS actuators for MultipleZone accim."""
    actuatorlist = ([actuator.Name for actuator in self.idf1.idfobjects['EnergyManagementSystem:Actuator']])

    for zonename in self.zonenames:
        if 'FORSCRIPT_AHST_Schedule_'+zonename in actuatorlist:
            if verboseMode:
                print('Not added - FORSCRIPT_AHST_Sch_'+zonename+' Actuator')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Actuator',
                Name='FORSCRIPT_AHST_Sch_'+zonename,
                Actuated_Component_Unique_Name='FORSCRIPT_AHST_'+zonename,
                Actuated_Component_Type='Schedule:Compact',
                Actuated_Component_Control_Type='Schedule Value'
                )
            if verboseMode:
                print('Added - FORSCRIPT_AHST_Sch_'+zonename+' Actuator')
        #    print([actuator for actuator in self.idf1.idfobjects['EnergyManagementSystem:Actuator'] if actuator.Name=='FORSCRIPT_AHST_Schedule_'+zonename])

        if 'FORSCRIPT_ACST_Schedule_'+zonename in actuatorlist:
            if verboseMode:
                print('Not added - FORSCRIPT_ACST_Sch_'+zonename+' Actuator')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Actuator',
                Name='FORSCRIPT_ACST_Sch_'+zonename,
                Actuated_Component_Unique_Name='FORSCRIPT_ACST_'+zonename,
                Actuated_Component_Type='Schedule:Compact',
                Actuated_Component_Control_Type='Schedule Value'
                )
            if verboseMode:
                print('Added - FORSCRIPT_ACST_Sch_'+zonename+' Actuator')
        #    print([actuator for actuator in self.idf1.idfobjects['EnergyManagementSystem:Actuator'] if actuator.Name=='FORSCRIPT_ACST_Schedule_'+zonename])

    for i in range(len(self.windownamelist)):
        if self.windownamelist[i]+'_VentOpenFact' in actuatorlist:
            if verboseMode:
                print('Not added - '+self.windownamelist[i]+'_OpT Actuator')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Actuator',
                Name=self.windownamelist[i]+'_VentOpenFact',
                Actuated_Component_Unique_Name=self.windownamelist_orig[i],
                Actuated_Component_Type='AirFlow Network Window/Door Opening',
                Actuated_Component_Control_Type='Venting Opening Factor'
                )
            if verboseMode:
                print('Added - '+self.windownamelist[i]+'_VentOpenFact Actuator')
    del actuatorlist


def addEMSProgramsMultipleZone(self, verboseMode: bool = True):
    """Add EMS programs for MultipleZone accim."""
    programlist = ([program.Name for program in self.idf1.idfobjects['EnergyManagementSystem:Program']])

    if 'SetInputData' in programlist:
        if verboseMode:
            print('Not added - SetInputData Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='SetInputData',
            Program_Line_1='set AdapStand = 1',
            Program_Line_2='set CAT = 1',
            Program_Line_3='set ComfMod = 2',
            Program_Line_4='set HVACmode = 2',
            Program_Line_5='set VentCtrl = 0',
            Program_Line_6='set VSToffset = 0',
            Program_Line_7='set MinOToffset = 7',
            Program_Line_8='set MaxWindSpeed = 6',
            Program_Line_9='set ACSTtol = -0.25',
            Program_Line_10='set AHSTtol = 0.25'
            )
        if verboseMode:
            print('Added - SetInputData Program')

    if 'SetVST' in programlist:
        if verboseMode:
            print('Not added - SetVST Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='SetVST',
            Program_Line_1='set MinOutTemp = AHST - MinOToffset',
            Program_Line_2='if AdapStand == 0',
            Program_Line_3='if (CurrentTime < 7)',
            Program_Line_4='set VST = (ACST+AHST)/2+VSToffset',
            Program_Line_5='elseif (CurrentTime < 15)',
            Program_Line_6='set VST = 22.5+VSToffset',
            Program_Line_7='elseif (CurrentTime < 23)',
            Program_Line_8='set VST = (ACST+AHST)/2+VSToffset',
            Program_Line_9='elseif (CurrentTime < 24)',
            Program_Line_10='set VST = (ACST+AHST)/2+VSToffset',
            Program_Line_11='endif',
            Program_Line_12='elseif AdapStand == 1',
            Program_Line_13='if (RMOT >= AHSTall) && (RMOT <= ACSTaul)',
            Program_Line_14='set VST = ComfTemp+VSToffset',
            Program_Line_15='else',
            Program_Line_16='set VST = (ACST+AHST)/2+VSToffset',
            Program_Line_17='endif',
            Program_Line_18='elseif AdapStand == 2',
            Program_Line_19='if (PMOT >= AHSTall) && (PMOT <= ACSTaul)',
            Program_Line_20='set VST = ComfTemp+VSToffset',
            Program_Line_21='else',
            Program_Line_22='set VST = (ACST+AHST)/2+VSToffset',
            Program_Line_23='endif',
            Program_Line_24='endif'
            )
        if verboseMode:
            print('Added - SetVST Program')
    #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetVST'])

    for zonename in self.zonenames:
        if 'ApplyAST_MultipleZone_'+zonename in programlist:
            if verboseMode:
                print('Not added - ApplyAST_MultipleZone_'+zonename+' Program')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Program',
                Name='ApplyAST_MultipleZone_'+zonename,
                Program_Line_1='if ('+zonename+'_OpT>VST)&&('+zonename+'_OutT < VST)',
                Program_Line_2='if '+zonename+'_CoolCoil==0',
                Program_Line_3='if '+zonename+'_HeatCoil==0',
                Program_Line_4='if ('+zonename+'_OpT<ACST)&&('+zonename+'_OutT>MinOutTemp)',
                Program_Line_5='if '+zonename+'_WindSpeed <= MaxWindSpeed',
                Program_Line_6='set Ventilates_HVACmode2_'+zonename+' = 1',
                Program_Line_7='else',
                Program_Line_8='set Ventilates_HVACmode2_'+zonename+' = 0',
                Program_Line_9='endif',
                Program_Line_10='else',
                Program_Line_11='set Ventilates_HVACmode2_'+zonename+' = 0',
                Program_Line_12='endif',
                Program_Line_13='else',
                Program_Line_14='set Ventilates_HVACmode2_'+zonename+' = 0',
                Program_Line_15='endif',
                Program_Line_16='else',
                Program_Line_17='set Ventilates_HVACmode2_'+zonename+' = 0',
                Program_Line_18='endif',
                Program_Line_19='else',
                Program_Line_20='set Ventilates_HVACmode2_'+zonename+' = 0',
                Program_Line_21='endif',
                Program_Line_22='if VentCtrl == 0',
                Program_Line_23='if '+zonename+'_OutT < '+zonename+'_OpT',
                Program_Line_24='if '+zonename+'_OutT>MinOutTemp',
                Program_Line_25='if '+zonename+'_OpT > VST',
                Program_Line_26='if '+zonename+'_WindSpeed <= MaxWindSpeed',
                Program_Line_27='set Ventilates_HVACmode1_'+zonename+' = 1',
                Program_Line_28='else',
                Program_Line_29='set Ventilates_HVACmode1_'+zonename+' = 0',
                Program_Line_30='endif',
                Program_Line_31='else',
                Program_Line_32='set Ventilates_HVACmode1_'+zonename+' = 0',
                Program_Line_33='endif',
                Program_Line_34='else',
                Program_Line_35='set Ventilates_HVACmode1_'+zonename+' = 0',
                Program_Line_36='endif',
                Program_Line_37='else',
                Program_Line_38='set Ventilates_HVACmode1_'+zonename+' = 0',
                Program_Line_39='endif',
                Program_Line_40='elseif VentCtrl == 1',
                Program_Line_41='if '+zonename+'_OutT<'+zonename+'_OpT',
                Program_Line_42='if '+zonename+'_OutT>MinOutTemp',
                Program_Line_43='if '+zonename+'_OpT > ACSTnoTol',
                Program_Line_44='if '+zonename+'_WindSpeed <= MaxWindSpeed',
                Program_Line_45='set Ventilates_HVACmode1_'+zonename+' = 1',
                Program_Line_46='else',
                Program_Line_47='set Ventilates_HVACmode1_'+zonename+' = 0',
                Program_Line_48='endif',
                Program_Line_49='else',
                Program_Line_50='set Ventilates_HVACmode1_'+zonename+' = 0',
                Program_Line_51='endif',
                Program_Line_52='else',
                Program_Line_53='set Ventilates_HVACmode1_'+zonename+' = 0',
                Program_Line_54='endif',
                Program_Line_55='else',
                Program_Line_56='set Ventilates_HVACmode1_'+zonename+' = 0',
                Program_Line_57='endif',
                Program_Line_58='endif',
                Program_Line_59='if HVACmode == 0',
                Program_Line_60='set FORSCRIPT_ACST_Sch_'+zonename+' = ACST',
                Program_Line_61='set FORSCRIPT_AHST_Sch_'+zonename+' = AHST',
                Program_Line_62='elseif HVACmode == 1',
                Program_Line_63='Set FORSCRIPT_ACST_Sch_'+zonename+' = 100',
                Program_Line_64='Set FORSCRIPT_AHST_Sch_'+zonename+' = -100',
                Program_Line_65='elseif HVACmode == 2',
                Program_Line_66='if Ventilates_HVACmode2_'+zonename+' == 0',
                Program_Line_67='set FORSCRIPT_ACST_Sch_'+zonename+' = ACST',
                Program_Line_68='set FORSCRIPT_AHST_Sch_'+zonename+' = AHST',
                Program_Line_69='endif',
                Program_Line_70='endif'
                )
            if verboseMode:
                print('Added - ApplyAST_MultipleZone_'+zonename+' Program')
        #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'ApplyAST_MultipleZone_'+windowname])

    for windowname in self.windownamelist:
        if 'SetWindowOperation_'+windowname in programlist:
            if verboseMode:
                print('Not added - SetWindowOperation_'+windowname+' Program')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Program',
                Name='SetWindowOperation_'+windowname,
                Program_Line_1='if ('+windowname+'_OpT>VST)&&('+windowname+'_OutT < VST)',
                Program_Line_2='if '+windowname+'_CoolCoil==0',
                Program_Line_3='if '+windowname+'_HeatCoil==0',
                Program_Line_4='if ('+windowname+'_OpT<ACST)&&('+windowname+'_OutT>MinOutTemp)',
                Program_Line_5='if '+windowname+'_WindSpeed <= MaxWindSpeed',
                Program_Line_6='set Ventilates_HVACmode2_'+windowname+' = 1',
                Program_Line_7='else',
                Program_Line_8='set Ventilates_HVACmode2_'+windowname+' = 0',
                Program_Line_9='endif',
                Program_Line_10='else',
                Program_Line_11='set Ventilates_HVACmode2_'+windowname+' = 0',
                Program_Line_12='endif',
                Program_Line_13='else',
                Program_Line_14='set Ventilates_HVACmode2_'+windowname+' = 0',
                Program_Line_15='endif',
                Program_Line_16='else',
                Program_Line_17='set Ventilates_HVACmode2_'+windowname+' = 0',
                Program_Line_18='endif',
                Program_Line_19='else',
                Program_Line_20='set Ventilates_HVACmode2_'+windowname+' = 0',
                Program_Line_21='endif',
                Program_Line_22='if VentCtrl == 0',
                Program_Line_23='if '+windowname+'_OutT < '+windowname+'_OpT',
                Program_Line_24='if '+windowname+'_OutT>MinOutTemp',
                Program_Line_25='if '+windowname+'_OpT > VST',
                Program_Line_26='if '+windowname+'_WindSpeed <= MaxWindSpeed',
                Program_Line_27='set Ventilates_HVACmode1_'+windowname+' = 1',
                Program_Line_28='else',
                Program_Line_29='set Ventilates_HVACmode1_'+windowname+' = 0',
                Program_Line_30='endif',
                Program_Line_31='else',
                Program_Line_32='set Ventilates_HVACmode1_'+windowname+' = 0',
                Program_Line_33='endif',
                Program_Line_34='else',
                Program_Line_35='set Ventilates_HVACmode1_'+windowname+' = 0',
                Program_Line_36='endif',
                Program_Line_37='else',
                Program_Line_38='set Ventilates_HVACmode1_'+windowname+' = 0',
                Program_Line_39='endif',
                Program_Line_40='elseif VentCtrl == 1',
                Program_Line_41='if '+windowname+'_OutT<'+windowname+'_OpT',
                Program_Line_42='if '+windowname+'_OutT>MinOutTemp',
                Program_Line_43='if '+windowname+'_OpT > ACSTnoTol',
                Program_Line_44='if '+windowname+'_WindSpeed <= MaxWindSpeed',
                Program_Line_45='set Ventilates_HVACmode1_'+windowname+' = 1',
                Program_Line_46='else',
                Program_Line_47='set Ventilates_HVACmode1_'+windowname+' = 0',
                Program_Line_48='endif',
                Program_Line_49='else',
                Program_Line_50='set Ventilates_HVACmode1_'+windowname+' = 0',
                Program_Line_51='endif',
                Program_Line_52='else',
                Program_Line_53='set Ventilates_HVACmode1_'+windowname+' = 0',
                Program_Line_54='endif',
                Program_Line_55='else',
                Program_Line_56='set Ventilates_HVACmode1_'+windowname+' = 0',
                Program_Line_57='endif',
                Program_Line_58='endif',
                Program_Line_59='if HVACmode == 0',
                Program_Line_60='set '+windowname+'_VentOpenFact = 0',
                Program_Line_61='elseif HVACmode == 1',
                Program_Line_62='if Ventilates_HVACmode1_'+windowname+' == 1',
                Program_Line_63='set '+windowname+'_VentOpenFact = 1',
                Program_Line_64='else',
                Program_Line_65='set '+windowname+'_VentOpenFact = 0',
                Program_Line_66='endif',
                Program_Line_67='elseif HVACmode == 2',
                Program_Line_68='if Ventilates_HVACmode2_'+windowname+' == 1',
                Program_Line_69='set '+windowname+'_VentOpenFact = 1',
                Program_Line_70='else',
                Program_Line_71='set '+windowname+'_VentOpenFact = 0',
                Program_Line_72='endif',
                Program_Line_73='endif'
                )
            if verboseMode:
                print('Added - SetWindowOperation_'+windowname+' Program')
        #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetWindowOperation_'+windowname])

    del programlist


def addEMSOutputVariableMultipleZone(self, verboseMode: bool = True):
    """Add EMS output variables for MultipleZone accim."""
    outputvariablelist = ([program.Name for program in self.idf1.idfobjects['EnergyManagementSystem:OutputVariable']])

    if 'Ventilation Setpoint Temperature' in outputvariablelist:
        if verboseMode:
            print('Not added - Ventilation Setpoint Temperature Output Variable')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:OutputVariable',
            Name='Ventilation Setpoint Temperature',
            EMS_Variable_Name='VST',
            Type_of_Data_in_Variable='Averaged',
            Update_Frequency='ZoneTimestep',
            EMS_Program_or_Subroutine_Name='',
            Units='C'
            )
        if verboseMode:
            print('Added - Ventilation Setpoint Temperature Output Variable')
    #    print([outputvariable for outputvariable in self.idf1.idfobjects['EnergyManagementSystem:OutputVariable'] if outputvariable.Name == 'Ventilation Setpoint Temperature'])

    if 'Minimum Outdoor Temperature for MultipleZone ventilation' in outputvariablelist:
        if verboseMode:
            print('Not added - Minimum Outdoor Temperature for MultipleZone ventilation Output Variable')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:OutputVariable',
            Name='Minimum Outdoor Temperature for MultipleZone ventilation',
            EMS_Variable_Name='MinOutTemp',
            Type_of_Data_in_Variable='Averaged',
            Update_Frequency='ZoneTimestep',
            EMS_Program_or_Subroutine_Name='',
            Units='C'
            )
        if verboseMode:
            print('Added - Ventilation Setpoint Temperature Output Variable')
    #    print([outputvariable for outputvariable in self.idf1.idfobjects['EnergyManagementSystem:OutputVariable'] if outputvariable.Name == 'Minimum Outdoor Temperature for MultipleZone ventilation'])
    del outputvariablelist


def addOutputVariablesMultipleZone(self, verboseMode: bool = True):
    """Add Output:Variable objects for MultipleZone accim."""
    EnvironmentalImpactFactorslist = ([output for output in self.idf1.idfobjects['Output:EnvironmentalImpactFactors']])
    for i in range(len(EnvironmentalImpactFactorslist)):
        firstEnvironmentalImpactFactor = self.idf1.idfobjects['Output:EnvironmentalImpactFactors'][-1]
        self.idf1.removeidfobject(firstEnvironmentalImpactFactor)

    outputmeterlist = ([output for output in self.idf1.idfobjects['Output:Meter']])
    for i in range(len(outputmeterlist)):
        firstoutputmeter = self.idf1.idfobjects['Output:Meter'][-1]
        self.idf1.removeidfobject(firstoutputmeter)

    alloutputs = ([output for output in self.idf1.idfobjects['Output:Variable']])
    for i in range(len(alloutputs)):
        firstoutput = self.idf1.idfobjects['Output:Variable'][-1]
        self.idf1.removeidfobject(firstoutput)

    # del EnvironmentalImpactFactorslist,firstEnvironmentalImpactFactor, outputmeterlist, firstoutputmeter, alloutputs, firstoutput

    outputvariablelist = ([outputvariable.Name for outputvariable in self.idf1.idfobjects['EnergyManagementSystem:OutputVariable']])
    outputlist = ([output.Variable_Name for output in self.idf1.idfobjects['Output:Variable']])
    addittionaloutputs = [
        'Zone Thermostat Operative Temperature',
        'Zone Thermal Comfort CEN 15251 Adaptive Model Running Average Outdoor Air Temperature',
        'Cooling Coil Total Cooling Rate',
        'Heating Coil Heating Rate',
        'Facility Total HVAC Electric Demand Power',
        'Facility Total HVAC Electricity Demand Rate',
        'VRF Heat Pump Cooling Electricity Energy',
        'VRF Heat Pump Heating Electricity Energy',
        'AFN Surface Venting Window or Door Opening Factor',
        'AFN Zone Infiltration Air Change Rate',
        'AFN Zone Infiltration Volume'
        ]

    for outputvariable in outputvariablelist:
        if outputvariable in outputlist:
            if verboseMode:
                print('Not added - '+outputvariable+' Output:Variable data')
        elif outputvariable.startswith("WIP"):
            if verboseMode:
                print('Not added - '+outputvariable+' Output:Variable data because its WIP')
        elif outputvariable.startswith('Adaptive Thermal Comfort Cost Index'):
            if verboseMode:
                print('Not added - '+outputvariable+' Output:Variable data because its ATCCI')
        else:
            self.idf1.newidfobject(
                'Output:Variable',
                Key_Value='*',
                Variable_Name=outputvariable,
                Reporting_Frequency='Hourly',
                Schedule_Name=''
                )
            if verboseMode:
                print('Added - '+outputvariable+' Output:Variable data')
    #        print([output for output in self.idf1.idfobjects['Output:Variable'] if output.Variable_Name == outputvariable])

    for addittionaloutput in addittionaloutputs:
        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value='*',
            Variable_Name=addittionaloutput,
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - '+addittionaloutput+' Output:Variable data')

    del outputvariablelist, outputlist, addittionaloutputs,

    self.idf1.newidfobject(
        'Output:Variable',
        Key_Value='Environment',
        Variable_Name='Site Outdoor Air Drybulb Temperature',
        Reporting_Frequency='Hourly',
        Schedule_Name=''
        )
    if verboseMode:
        print('Added - Site Outdoor Air Drybulb Temperature Output:Variable data')

    zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
    for zonename in zonenames:
        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value='FORSCRIPT_AHST_'+zonename,
            Variable_Name='Schedule Value',
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - FORSCRIPT_AHST_'+zonename+' Output:Variable data')

        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value='FORSCRIPT_ACST_'+zonename,
            Variable_Name='Schedule Value',
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - FORSCRIPT_ACST_'+zonename+' Output:Variable data')

    for zonename in self.zonenames_orig:
        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value=zonename,
            Variable_Name='Zone Operative Temperature',
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - '+zonename+' Zone Operative Temperature Output:Variable data')

        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value=zonename+' VRF Indoor Unit DX Cooling Coil',
            Variable_Name='Cooling Coil Total Cooling Rate',
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - '+zonename+' VRF Indoor Unit DX Cooling Coil Output:Variable data')

        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value=zonename+' VRF Indoor Unit DX Heating Coil',
            Variable_Name='Heating Coil Heating Rate',
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - '+zonename+' VRF Indoor Unit DX Heating Coil Output:Variable data')

        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value=zonename,
            Variable_Name='Zone Outdoor Air Wind Speed',
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - '+zonename+' Zone Outdoor Air Wind Speed Output:Variable data')



"""Class for accim SingleZone and MultipleZone."""

class accimJob():
    """Class to start the accim process."""
    # todo clean imports
    from os import listdir
    import numpy

    from accim.sim.accim_IDFgeneration import \
        inputdataSingleZone,\
        inputdataMultipleZone, \
        genIDFSingleZone, \
        genIDFMultipleZone
    from accim.sim.accim_Base import \
        setComfFieldsPeople, \
        addOpTempTherm, \
        addBaseSchedules, \
        setAvailSchOn, \
        saveaccim
    from accim.sim.accim_Base_EMS import \
        addEMSProgramsBase, \
        addEMSPCMBase, \
        addEMSOutputVariableBase, \
        addOutputVariablesTimestep, \
        addSimplifiedOutputVariables
    from accim.sim.accim_SingleZone import \
        addForscriptSchSingleZone
    from accim.sim.accim_SingleZone_EMS import \
        addGlobVarListSingleZone, \
        addEMSSensorsSingleZone, \
        addEMSActuatorsSingleZone, \
        addEMSProgramsSingleZone, \
        addOutputVariablesSingleZone
    from accim.sim.accim_MultipleZone import \
        addMultipleZoneSch, \
        addCurveObj, \
        addDetHVACobjEp91, \
        addDetHVACobjEp94, \
        addForscriptSchMultipleZone, \
        checkVentIsOn
    from accim.sim.accim_MultipleZone_EMS import \
        addGlobVarListMultipleZone, \
        addEMSSensorsMultipleZone, \
        addEMSActuatorsMultipleZone, \
        addEMSProgramsMultipleZone, \
        addEMSOutputVariableMultipleZone, \
        addOutputVariablesMultipleZone

    def __init__(self, filename_temp, ScriptType: str = None, EnergyPlus_version: str = None, verboseMode: bool = True):
        from eppy import modeleditor
        from eppy.modeleditor import IDF
        if EnergyPlus_version.lower() == 'ep94':
            iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
        elif EnergyPlus_version.lower() == 'ep91':
            iddfile = 'C:/EnergyPlusV9-1-0/Energy+.idd'
        else:
            raise ValueError("""EnergyPlus version not supported.\n
                                     Only works for EnergyPlus 9.1 (enter Ep91) and
                                     EnergyPlus 9.4 (enter Ep94) versions.""")
        IDF.setiddname(iddfile)

        fname1 = filename_temp+'.idf'
        self.idf0 = IDF(fname1)
        self.idf0.savecopy(filename_temp+'_pymod.idf')

        self.filename = filename_temp+'_pymod'
        fname1 = self.filename+'.idf'
        self.idf1 = IDF(fname1)
        self.filename = filename_temp+'_pymod'

        # print(self.filename)

        self.zonenames_orig = ([zone.Name for zone in self.idf1.idfobjects['ZONE']])
        # print(self.zonenames_orig)

        self.zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
        # print(self.zonenames)
        if verboseMode:
            print(f'The zones in the model {filename_temp} are:')
            print(*self.zonenames, sep="\n")

        if ScriptType.lower() == 'mz' or ScriptType.lower() == 'multiplezone':
            self.windownamelist_orig = ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])
            # print(self.windownamelist_orig)
            self.windownamelist_orig_split = ([i.split('_') for i in self.windownamelist_orig])
            # print(self.windownamelist_orig_split)

            self.windownamelist = ([sub.replace(':', '_') for sub in ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])])
            # print(self.windownamelist)
            if verboseMode:
                print(f'The windows in the model {filename_temp} are:')
                print(*self.windownamelist, sep="\n")



"""Add EMS objects only for SingleZone mode."""


def addGlobVarListSingleZone(self, verboseMode: bool = True):
    """
    Amend Global Variable objects for SingleZone accim to work.

    Remove existing Global Variable objects and
    add correct Global Variable objects for SingleZone accim.
    """
    globalvariablelist = ([program for program in self.idf1.idfobjects['ENERGYMANAGEMENTSYSTEM:GLOBALVARIABLE']])

    for i in range(len(globalvariablelist)):
        firstglobalvariablelist = self.idf1.idfobjects['ENERGYMANAGEMENTSYSTEM:GLOBALVARIABLE'][-1]
        self.idf1.removeidfobject(firstglobalvariablelist)

    del globalvariablelist

    self.idf1.newidfobject(
        'EnergyManagementSystem:GlobalVariable',
        Erl_Variable_1_Name='ACST',
        Erl_Variable_2_Name='AHST',
        Erl_Variable_3_Name='ACSTnoTol',
        Erl_Variable_4_Name='AHSTnoTol',
        Erl_Variable_5_Name='AdapStand',
        Erl_Variable_6_Name='ACSTaul',
        Erl_Variable_7_Name='ACSTall',
        Erl_Variable_8_Name='AHSTaul',
        Erl_Variable_9_Name='AHSTall',
        Erl_Variable_10_Name='CAT',
        Erl_Variable_11_Name='ACSToffset',
        Erl_Variable_12_Name='AHSToffset',
        Erl_Variable_13_Name='ComfMod',
        Erl_Variable_14_Name='ComfTemp',
        Erl_Variable_15_Name='ACSTtol',
        Erl_Variable_16_Name='AHSTtol'
        )

    for zonename in self.zonenames:
        self.idf1.newidfobject(
            'EnergyManagementSystem:GlobalVariable',
            Erl_Variable_1_Name='ComfHours_'+zonename,
            Erl_Variable_2_Name='DiscomfAppHotHours_'+zonename,
            Erl_Variable_3_Name='DiscomfAppColdHours_'+zonename,
            Erl_Variable_4_Name='DiscomfNonAppHotHours_'+zonename,
            Erl_Variable_5_Name='DiscomfNonAppColdHours_'+zonename,
            Erl_Variable_6_Name='ComfHoursNoApp_'+zonename
            )
    if verboseMode:
        print("Global variables objects have been added")


def addEMSSensorsSingleZone(self, verboseMode: bool = True):
    """Add EMS sensors for SingleZone accim."""
    if 'RMOT' in [sensor.Name for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor']]:
        if verboseMode:
            print('Not added - RMOT Sensor')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Sensor',
            Name='RMOT',
            OutputVariable_or_OutputMeter_Index_Key_Name='People '+self.zonenames_orig[0],
            OutputVariable_or_OutputMeter_Name='Zone Thermal Comfort CEN 15251 Adaptive Model Running Average Outdoor Air Temperature'
            )
        if verboseMode:
            print('Added - RMOT Sensor')
    #    print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name=='RMOT'])

    if 'PMOT' in [sensor.Name for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor']]:
        if verboseMode:
            print('Not added - PMOT Sensor')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Sensor',
            Name='PMOT',
            OutputVariable_or_OutputMeter_Index_Key_Name='People '+self.zonenames_orig[0],
            OutputVariable_or_OutputMeter_Name='Zone Thermal Comfort ASHRAE 55 Adaptive Model Running Average Outdoor Air Temperature'
            )
        if verboseMode:
            print('Added - PMOT Sensor')
    #    print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name=='PMOT'])

    for i in range(len(self.zonenames)):
        if self.zonenames[i]+'_OpT' in [sensor.Name for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor']]:
            if verboseMode:
                print('Not added - '+self.zonenames[i]+'_OpT Sensor')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Sensor',
                Name=self.zonenames[i]+'_OpT',
                OutputVariable_or_OutputMeter_Index_Key_Name=self.zonenames_orig[i],
                OutputVariable_or_OutputMeter_Name='Zone Operative Temperature'
                )
            if verboseMode:
                print('Added - '+self.zonenames[i]+'_OpT Sensor')
    #        print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name==self.zonenames[i]+'_OpT'])

    # if 'HVACConsump' in [sensor.Name for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor']]:
    #     print('Not added - HVACConsump Sensor')
    # else:
    #     self.idf1.newidfobject(
    #         'EnergyManagementSystem:Sensor',
    #         Name='HVACConsump',
    #         OutputVariable_or_OutputMeter_Index_Key_Name='Whole Building',
    #         OutputVariable_or_OutputMeter_Name='Facility Total HVAC Electric Demand Power'
    #         )
    #     print('Added - HVACConsump Sensor')
    #     print([sensor for sensor in self.idf1.idfobjects['EnergyManagementSystem:Sensor'] if sensor.Name=='HVACConsump'])


def addEMSActuatorsSingleZone(self, verboseMode: bool = True):
    """Add EMS actuators for SingleZone accim."""
    actuatorlist = ([actuator.Name for actuator in self.idf1.idfobjects['EnergyManagementSystem:Actuator']])

    if 'FORSCRIPT_AHST_Schedule' in actuatorlist:
        if verboseMode:
            print('Not added - FORSCRIPT_AHST_Schedule Actuator')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Actuator',
            Name='FORSCRIPT_AHST_Schedule',
            Actuated_Component_Unique_Name='FORSCRIPT_AHST',
            Actuated_Component_Type='Schedule:Compact',
            Actuated_Component_Control_Type='Schedule Value'
            )
        if verboseMode:
            print('Added - FORSCRIPT_AHST_Schedule Actuator')
    #    print([actuator for actuator in self.idf1.idfobjects['EnergyManagementSystem:Actuator'] if actuator.Name=='FORSCRIPT_AHST_Schedule'])

    if 'FORSCRIPT_ACST_Schedule' in actuatorlist:
        if verboseMode:
            print('Not added - FORSCRIPT_ACST_Schedule Actuator')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Actuator',
            Name='FORSCRIPT_ACST_Schedule',
            Actuated_Component_Unique_Name='FORSCRIPT_ACST',
            Actuated_Component_Type='Schedule:Compact',
            Actuated_Component_Control_Type='Schedule Value'
            )
        if verboseMode:
            print('Added - FORSCRIPT_ACST_Schedule Actuator')
    #    print([actuator for actuator in self.idf1.idfobjects['EnergyManagementSystem:Actuator'] if actuator.Name=='FORSCRIPT_ACST_Schedule'])
    del actuatorlist


def addEMSProgramsSingleZone(self, verboseMode: bool = True):
    """Add EMS programs for SingleZone accim."""
    programlist = ([program.Name for program in self.idf1.idfobjects['EnergyManagementSystem:Program']])

    if 'SetInputData' in programlist:
        if verboseMode:
            print('Not added - SetInputData Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='SetInputData',
            Program_Line_1='set AdapStand = 1',
            Program_Line_2='set CAT = 1',
            Program_Line_3='set ComfMod = 1',
            Program_Line_4='set ACSTtol = -0.25',
            Program_Line_5='set AHSTtol = 0.25'
            )
        if verboseMode:
            print('Added - SetInputData Program')
        # print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetInputData'])

    if 'ApplyAST' in programlist:
        if verboseMode:
            print('Not added - ApplyAST Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='ApplyAST',
            Program_Line_1='Set FORSCRIPT_ACST_Schedule = ACST',
            Program_Line_2='Set FORSCRIPT_AHST_Schedule = AHST'
            )
        if verboseMode:
            print('Added - ApplyAST Program')
    #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'ApplyAST'])

    del programlist


def addOutputVariablesSingleZone(self, verboseMode: bool = True):
    """Add Output:Variable objects for SingleZone accim."""
    EnvironmentalImpactFactorslist = ([output for output in self.idf1.idfobjects['Output:EnvironmentalImpactFactors']])

    for i in range(len(EnvironmentalImpactFactorslist)):
        firstEnvironmentalImpactFactor = self.idf1.idfobjects['Output:EnvironmentalImpactFactors'][-1]
        self.idf1.removeidfobject(firstEnvironmentalImpactFactor)

    outputmeterlist = ([output for output in self.idf1.idfobjects['Output:Meter']])
    for i in range(len(outputmeterlist)):
        firstoutputmeter = self.idf1.idfobjects['Output:Meter'][-1]
        self.idf1.removeidfobject(firstoutputmeter)

    alloutputs = ([output for output in self.idf1.idfobjects['Output:Variable']])
    for i in range(len(alloutputs)):
        firstoutput = self.idf1.idfobjects['Output:Variable'][-1]
        self.idf1.removeidfobject(firstoutput)

    # del EnvironmentalImpactFactorslist,firstEnvironmentalImpactFactor, outputmeterlist, firstoutputmeter, alloutputs, firstoutput

    outputvariablelist = ([program.Name for program in self.idf1.idfobjects['EnergyManagementSystem:OutputVariable']])
    outputlist = ([output.Variable_Name for output in self.idf1.idfobjects['Output:Variable']])
    addittionaloutputs = [
        'Zone Thermostat Operative Temperature',
        'Zone Thermal Comfort CEN 15251 Adaptive Model Running Average Outdoor Air Temperature',
        'Cooling Coil Total Cooling Rate',
        'Heating Coil Heating Rate',
        'Facility Total HVAC Electric Demand Power',
        'Facility Total HVAC Electricity Demand Rate',
        'VRF Heat Pump Cooling Electricity Energy',
        'VRF Heat Pump Heating Electricity Energy'
        ]

    for outputvariable in outputvariablelist:
        if outputvariable in outputlist:
            if verboseMode:
                print('Not added - '+outputvariable+' Output:Variable data')
        elif outputvariable.startswith("WIP"):
            if verboseMode:
                print('Not added - '+outputvariable+' Output:Variable data because its WIP')
        elif outputvariable.startswith('Adaptive Thermal Comfort Cost Index'):
            if verboseMode:
                print('Not added - '+outputvariable+' Output:Variable data because its ATCCI')
        else:
            self.idf1.newidfobject(
                'Output:Variable',
                Key_Value='*',
                Variable_Name=outputvariable,
                Reporting_Frequency='Hourly',
                Schedule_Name=''
                )
            if verboseMode:
                print('Added - '+outputvariable+' Output:Variable data')
    #        print([output for output in self.idf1.idfobjects['Output:Variable'] if output.Variable_Name == outputvariable])

    for addittionaloutput in addittionaloutputs:
        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value='*',
            Variable_Name=addittionaloutput,
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - '+addittionaloutput+' Output:Variable data')

    del outputvariablelist, outputlist, addittionaloutputs,

    self.idf1.newidfobject(
        'Output:Variable',
        Key_Value='Environment',
        Variable_Name='Site Outdoor Air Drybulb Temperature',
        Reporting_Frequency='Hourly',
        Schedule_Name=''
        )
    if verboseMode:
        print('Added - Site Outdoor Air Drybulb Temperature Output:Variable data')

    self.idf1.newidfobject(
        'Output:Variable',
        Key_Value='FORSCRIPT_AHST',
        Variable_Name='Schedule Value',
        Reporting_Frequency='Hourly',
        Schedule_Name=''
        )
    if verboseMode:
        print('Added - FORSCRIPT_AHST Output:Variable data')

    self.idf1.newidfobject(
        'Output:Variable',
        Key_Value='FORSCRIPT_ACST',
        Variable_Name='Schedule Value',
        Reporting_Frequency='Hourly',
        Schedule_Name=''
        )
    if verboseMode:
        print('Added - FORSCRIPT_ACST Output:Variable data')

    for zonename in self.zonenames_orig:
        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value=zonename,
            Variable_Name='Zone Operative Temperature',
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - '+zonename+' Zone Operative Temperature Output:Variable data')



import pytest
from accim.sim import accim_Main

def test_genIDFSingleZone():
    pass
    from eppy.modeleditor import IDF
    from os import listdir
    import numpy

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimJob(
        filename_temp='TestModel_SingleZone',
        ScriptType='sz',
        EnergyPlus_version='ep94',
        verboseMode=False)

    z.addEMSProgramsSingleZone(verboseMode=False)
    z.saveaccim(verboseMode=False)

    idf1 = IDF('TestModel_SingleZone_pymod.idf')
    SetInputData = (
        [program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetInputData'])

    z.genIDFSingleZone(
        AdapStand=[1, 2],
        CAT=[1, 80],
        ComfMod=[1, 80],
        ASTtol_start=0.1,
        ASTtol_end_input=0.2,
        ASTtol_steps=0.1,
        NameSuffix='whatever',
        verboseMode=False,
        confirmGen=True
    )

    filelist_pymod = ([file for file in listdir() if file.endswith('_pymod.idf')])
    filelist_pymod = ([file.split('.idf')[0] for file in filelist_pymod])
    print(filelist_pymod)

    AdapStand_List = [1]
    CAT_List = [1, 80]
    ComfMod_List = [1, 80]
    ASTtol_value_from = 0.1
    ASTtol_value_to = 0.2
    ASTtol_value_steps = 0.1
    suffix = 'whatever'

    for file in filelist_pymod:
        filename = file
        fname1 = filename + '.idf'
        idf1 = IDF(fname1)
        SetInputData = (
        [program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetInputData'])
        for AdapStand_value in AdapStand_List:
            assert SetInputData[0].Program_Line_1 == 'set AdapStand  =  ' + repr(AdapStand_value)
            if AdapStand_value == 0:
                assert SetInputData[0].Program_Line_2 == 'set CAT = 1'
                assert SetInputData[0].Program_Line_3 == 'set ComfMod = 0'
                for ASTtol_value in numpy.arange(ASTtol_value_from, ASTtol_value_to, ASTtol_value_steps):
                    assert SetInputData[0].Program_Line_4 == 'set ACSTtol = ' + repr(-ASTtol_value)
                    assert SetInputData[0].Program_Line_5 == 'set AHSTtol = ' + repr(ASTtol_value)
                    outputname = (
                            filename
                            + '[AS_CTE'
                            + '[CA_X'
                            + '[CM_X'
                            + '[AT_' + repr(ASTtol_value)
                            + suffix
                            + '.idf'
                    )
                    assert outputname == filename
                    # idf1.savecopy(outputname)
            elif AdapStand_value == 1:
                for CAT_value in CAT_List:
                    if CAT_value not in range(0, 4):
                        continue
                    else:
                        assert SetInputData[0].Program_Line_2 == 'set CAT = ' + repr(CAT_value)
                        for ComfMod_value in ComfMod_List:
                            assert SetInputData[0].Program_Line_3 == 'set ComfMod = ' + repr(ComfMod_value)
                            for ASTtol_value in numpy.arange(ASTtol_value_from, ASTtol_value_to,
                                                             ASTtol_value_steps):
                                assert SetInputData[0].Program_Line_4 == 'set ACSTtol = ' + repr(-ASTtol_value)
                                assert SetInputData[0].Program_Line_5 == 'set AHSTtol = ' + repr(ASTtol_value)
                                outputname = (
                                        filename
                                        + '[AS_EN16798'
                                        + '[CA_' + repr(CAT_value)
                                        + '[CM_' + repr(ComfMod_value)
                                        + '[AT_' + repr(ASTtol_value)
                                        + suffix
                                        + '.idf'
                                )
                                assert outputname == filename
                                # idf1.savecopy(outputname)
            elif AdapStand_value == 2:
                for CAT_value in CAT_List:
                    if CAT_value not in range(80, 91, 10):
                        continue
                    else:
                        assert SetInputData[0].Program_Line_2 == 'set CAT = ' + repr(CAT_value)
                        for ComfMod_value in ComfMod_List:
                            assert SetInputData[0].Program_Line_3 == 'set ComfMod = ' + repr(ComfMod_value)
                            for ASTtol_value in numpy.arange(ASTtol_value_from, ASTtol_value_to,
                                                             ASTtol_value_steps):
                                assert SetInputData[0].Program_Line_4 == 'set ACSTtol = ' + repr(-ASTtol_value)
                                assert SetInputData[0].Program_Line_5 == 'set AHSTtol = ' + repr(ASTtol_value)
                                outputname = (
                                        filename
                                        + '[AS_ASHRAE55'
                                        + '[CA_' + repr(CAT_value)
                                        + '[CM_' + repr(ComfMod_value)
                                        + '[AT_' + repr(ASTtol_value)
                                        + suffix
                                        + '.idf'
                                )
                                assert outputname == filename
                                # idf1.savecopy(outputname)



"""Class for accim SingleZone and MultipleZone."""

class accimJob():
    """accim object."""
    # todo clean imports
    from os import listdir
    import numpy

    import accim.sim.accim_IDFgeneration as IDFgen
    import accim.sim.accim_Base as base
    import accim.sim.accim_Base_EMS as base_EMS
    import accim.sim.accim_SingleZone as sz
    import accim.sim.accim_SingleZone_EMS as sz_EMS
    import accim.sim.accim_MultipleZone as mz
    import accim.sim.accim_MultipleZone_EMS as mz_EMS

    def __init__(self, filename_temp, ScriptType: str = None, EnergyPlus_version: str = None, verboseMode: bool = True):
        from eppy import modeleditor
        from eppy.modeleditor import IDF

        if EnergyPlus_version.lower() == 'ep94':
            iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
        elif EnergyPlus_version.lower() == 'ep91':
            iddfile = 'C:/EnergyPlusV9-1-0/Energy+.idd'
        else:
            raise ValueError("""EnergyPlus version not supported.\n
                                     Only works for EnergyPlus 9.1 (enter Ep91) and
                                     EnergyPlus 9.4 (enter Ep94) versions.""")
        IDF.setiddname(iddfile)

        fname1 = filename_temp+'.idf'
        self.idf0 = IDF(fname1)
        self.idf0.savecopy(filename_temp+'_pymod.idf')

        self.filename = filename_temp+'_pymod'
        fname1 = self.filename+'.idf'
        self.idf1 = IDF(fname1)
        self.filename = filename_temp+'_pymod'

        # print(self.filename)

        self.zonenames_orig = ([zone.Name for zone in self.idf1.idfobjects['ZONE']])
        # print(self.zonenames_orig)

        self.zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
        # print(self.zonenames)
        if verboseMode:
            print(f'The zones in the model {filename_temp} are:')
            print(*self.zonenames, sep="\n")

        if ScriptType.lower() == 'mz' or ScriptType.lower() == 'multiplezone':
            self.windownamelist_orig = ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])
            # print(self.windownamelist_orig)
            self.windownamelist_orig_split = ([i.split('_') for i in self.windownamelist_orig])
            # print(self.windownamelist_orig_split)

            self.windownamelist = ([sub.replace(':', '_') for sub in ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])])
            # print(self.windownamelist)
            if verboseMode:
                print(f'The windows in the model {filename_temp} are:')
                print(*self.windownamelist, sep="\n")

    def addAccis(
            ScriptType: str = None,
            Outputs: str = None,
            EnergyPlus_version: str = None,
            AdapStand: any = None,
            CAT: any = None,
            ComfMod: any = None,
            HVACmode: any = None,
            VentCtrl: any = None,
            VSToffset: any = [0],
            MinOToffset: any = [50],
            MaxWindSpeed: any = [50],
            ASTtol_start: float = 0.1,
            ASTtol_end_input: float = 0.1,
            ASTtol_steps: float = 0.1,
            NameSuffix: str = '',
            verboseMode: bool = True,
            confirmGen: bool = None):
        """
        Parameters
        :param ScriptType: The default is 'MultipleZones'. Can be 'MultipleZones'or
            'mz', or 'SingleZone' or 'sz'.
        :param Outputs: The default is 'Standard'. Can be 'Standard',
            'Simplified' or 'Timestep'.
        :param EnergyPlus_version: The default is 'Ep94'. Can be 'Ep91' or 'Ep94'.
        :param AdapStand: The default is None.(0 = CTE; 1 = EN16798-1; 2 = ASHRAE 55)
        :param CAT: The default is None.(1 = CAT I; 2 = CAT II; 3 = CAT III; 80 = 80% ACCEPT; 90 = 90% ACCEPT)
        :param ComfMod: The default is None.(0 = Static; 1 = OUT-CTE; 2 = OUT-SEN16798/SASHRAE55; 3 = OUT-AEN16798/AASHRAE55)
        :param HVACmode: The default is None.(0 = Fully Air-conditioned; 1 = Naturally ventilated; 2 = Mixed Mode)
        :param VentCtrl: (0 = Ventilates above neutral temperature; 1 = Ventilates above upper comfort limit)
        :param VSToffset: Please refer to documentation.
        :param MinOToffset: Please refer to documentation.
        :param MaxWindSpeed: Please refer to documentation.
        :param ASTtol_start: Please refer to documentation.
        :param ASTtol_end_input: Please refer to documentation.
        :param ASTtol_steps: Please refer to documentation.
        :param NameSuffix: Please refer to documentation.

        Exceptions
                DESCRIPTION. EnergyPlus version not supported.
            Only works for EnergyPlus 9.1 (enter Ep91) and
            EnergyPlus 9.4 (enter Ep94) versions.

        :return:
        """
        import accim.sim.accim_Main as main
        from os import listdir

        filelist = ([file for file in listdir() if file.endswith('.idf')
                     and not '_pymod' in file])

        filelist = ([file.split('.idf')[0] for file in filelist])

        objArgsDef = (
            ScriptType is not None,
            Outputs is not None,
            EnergyPlus_version is not None
        )

        fullScriptTypeList = ['MultipleZone',
                              'multiplezone',
                              'mz',
                              'SingleZone',
                              'singlezone',
                              'sz'
                              ]

        fullOutputsList = [
            'Standard',
            'standard',
            'Simplified',
            'simplified',
            'Timestep',
            'timestep'
        ]

        fullEPversionsList = [
            'Ep91',
            'ep91',
            'Ep94',
            'ep94'
        ]

        if all(objArgsDef):
            pass
        else:
            ScriptType = input("Enter the ScriptType (MultipleZone or mz, or SingleZone or sz): ")
            while ScriptType not in fullScriptTypeList:
                ScriptType = input("ScriptType was not correct. Please, enter the ScriptType (MultipleZone or mz, or SingleZone or sz): ")
            Outputs = input("Enter the Output (Standard, Simplified or Timestep): ")
            while Outputs not in fullOutputsList:
                Outputs = input("Output was not correct. Please, enter the Output (Standard, Simplified or Timestep): ")
            EnergyPlus_version = input("Enter the EnergyPlus version (Ep91 or Ep94): ")
            while EnergyPlus_version not in fullEPversionsList:
                EnergyPlus_version = input("EnergyPlus version was not correct. Please, enter the EnergyPlus version (Ep91 or Ep94): ")
        if verboseMode:
            print('ScriptType is: ' + ScriptType)
        if ScriptType not in fullScriptTypeList:
            print('Valid ScriptTypes: ')
            print(fullScriptTypeList)
            raise ValueError(ScriptType + " is not a valid ScriptType. You must choose a ScriptType from the list above.")
        if verboseMode:
            print('Outputs are: ' + Outputs)
        if Outputs not in fullOutputsList:
            print('Valid Outputs: ')
            print(fullOutputsList)
            raise ValueError(Outputs + " is not a valid Output. You must choose a Output from the list above.")
        if verboseMode:
            print('EnergyPlus version is: ' + EnergyPlus_version)
        if EnergyPlus_version not in fullEPversionsList:
            print('Valid EnergyPlus_version: ')
            print(fullEPversionsList)
            raise ValueError(EnergyPlus_version + " is not a valid EnergyPlus_version. You must choose a EnergyPlus_version from the list above.")

        for file in filelist:
            if verboseMode:
                print('''\n=======================START OF PROCESS=======================\n''')
                print('Starting with file:')
                print(file)
            z = main.accimJob(filename_temp=file, ScriptType=ScriptType, EnergyPlus_version=EnergyPlus_version, verboseMode=verboseMode)

            z.base.setComfFieldsPeople(verboseMode=verboseMode)
            z.base.addOpTempTherm(verboseMode=verboseMode)
            z.base.addBaseSchedules(verboseMode=verboseMode)
            z.base.setAvailSchOn(verboseMode=verboseMode)

            if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
                z.mz.addMultipleZoneSch(verboseMode=verboseMode)
                z.mz.addCurveObj(verboseMode=verboseMode)
                if EnergyPlus_version.lower() == 'ep94':
                    z.mz.addDetHVACobjEp94(verboseMode=verboseMode)
                elif EnergyPlus_version.lower() == 'ep91':
                    z.mz.addDetHVACobjEp91(verboseMode=verboseMode)
                z.mz.checkVentIsOn(verboseMode=verboseMode)
                z.mz.addForscriptSchMultipleZone(verboseMode=verboseMode)
            elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
                z.sz.addForscriptSchSingleZone(verboseMode=verboseMode)

            z.base_EMS.addEMSProgramsBase(verboseMode=verboseMode)
            z.base_EMS.addEMSOutputVariableBase(verboseMode=verboseMode)

            if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
                z.mz_EMS.addGlobVarListMultipleZone(verboseMode=verboseMode)
                z.mz_EMS.addEMSSensorsMultipleZone(verboseMode=verboseMode)
                z.mz_EMS.addEMSActuatorsMultipleZone(verboseMode=verboseMode)
                z.mz_EMS.addEMSProgramsMultipleZone(verboseMode=verboseMode)
            elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
                z.sz_EMS.addGlobVarListSingleZone(verboseMode=verboseMode)
                z.sz_EMS.addEMSSensorsSingleZone(verboseMode=verboseMode)
                z.sz_EMS.addEMSActuatorsSingleZone(verboseMode=verboseMode)
                z.sz_EMS.addEMSProgramsSingleZone(verboseMode=verboseMode)

            z.base_EMS.addEMSPCMBase(verboseMode=verboseMode)

            if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
                z.mz_EMS.addEMSOutputVariableMultipleZone(verboseMode=verboseMode)
                if Outputs.lower() == 'Simplified'.lower():
                    z.base_EMS.addSimplifiedOutputVariables(verboseMode=verboseMode)
                elif Outputs.lower() == 'Standard'.lower():
                    z.mz_EMS.addOutputVariablesMultipleZone(verboseMode=verboseMode)
            elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
                if Outputs.lower() == 'Simplified'.lower():
                    z.base_EMS.addSimplifiedOutputVariables(verboseMode=verboseMode)
                elif Outputs.lower() == 'Standard'.lower():
                    z.sz_EMS.addOutputVariablesSingleZone(verboseMode=verboseMode)
            if Outputs.lower() == 'Timestep'.lower():
                z.base_EMS.addOutputVariablesTimestep(verboseMode=verboseMode)

            z.base.saveaccim(verboseMode=verboseMode)
            if verboseMode:
                print('Ending with file:')
                print(file)
                print('''\n=======================END OF PROCESS=======================\n''')

        # arguments = (
        #     AdapStand is None,
        #     CAT is None,
        #     ComfMod is None,
        #     HVACmode is None,
        #     VentCtrl is None,
        #     VSToffset == [0],
        #     MinOToffset == [50],
        #     MaxWindSpeed == [50],
        #     ASTtol_start == 0.1,
        #     ASTtol_end_input == 0.1,
        #     ASTtol_steps == 0.1
        #     )

        args_needed_mz = (
            AdapStand is not None,
            CAT is not None,
            ComfMod is not None,
            HVACmode is not None,
            VentCtrl is not None,
        )

        args_needed_sz = (
            AdapStand is not None,
            CAT is not None,
            ComfMod is not None,
        )

        if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
            if all(args_needed_mz):
                z.IDFgen.genIDFMultipleZone(
                    AdapStand=AdapStand,
                    CAT=CAT,
                    ComfMod=ComfMod,
                    HVACmode=HVACmode,
                    VentCtrl=VentCtrl,
                    VSToffset=VSToffset,
                    MinOToffset=MinOToffset,
                    MaxWindSpeed=MaxWindSpeed,
                    ASTtol_start=ASTtol_start,
                    ASTtol_end_input=ASTtol_end_input,
                    ASTtol_steps=ASTtol_steps,
                    NameSuffix=NameSuffix,
                    verboseMode=verboseMode,
                    confirmGen=confirmGen
                )
            else:
                z.IDFgen.inputdataMultipleZone()
                z.IDFgen.genIDFMultipleZone()
        elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
            if all(args_needed_sz):
                z.IDFgen.genIDFSingleZone(
                    AdapStand=AdapStand,
                    CAT=CAT,
                    ComfMod=ComfMod,
                    ASTtol_start=ASTtol_start,
                    ASTtol_end_input=ASTtol_end_input,
                    ASTtol_steps=ASTtol_steps,
                    NameSuffix=NameSuffix,
                    verboseMode=verboseMode,
                    confirmGen=confirmGen
                )
            else:
                z.IDFgen.inputdataSingleZone()
                z.IDFgen.genIDFSingleZone()



import pytest
from accim.sim import accim_Main

@pytest.fixture()
def IDFobject():
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimJob(
        filename_temp='TestModel_SingleZone',
        ScriptType='sz',
        EnergyPlus_version='ep94',
        verboseMode=False)
    return z

def test_addGlobVarListSingleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addGlobVarListSingleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:GlobalVariable']])
    assert obj[0].Erl_Variable_1_Name == 'ACST'
    assert obj[0].Erl_Variable_2_Name == 'AHST'
    assert obj[0].Erl_Variable_3_Name == 'ACSTnoTol'
    assert obj[0].Erl_Variable_4_Name == 'AHSTnoTol'
    assert obj[0].Erl_Variable_5_Name == 'AdapStand'
    assert obj[0].Erl_Variable_6_Name == 'ACSTaul'
    assert obj[0].Erl_Variable_7_Name == 'ACSTall'
    assert obj[0].Erl_Variable_8_Name == 'AHSTaul'
    assert obj[0].Erl_Variable_9_Name == 'AHSTall'
    assert obj[0].Erl_Variable_10_Name == 'CAT'
    assert obj[0].Erl_Variable_11_Name == 'ACSToffset'
    assert obj[0].Erl_Variable_12_Name == 'AHSToffset'
    assert obj[0].Erl_Variable_13_Name == 'ComfMod'
    assert obj[0].Erl_Variable_14_Name == 'ComfTemp'
    assert obj[0].Erl_Variable_15_Name == 'ACSTtol'
    assert obj[0].Erl_Variable_16_Name == 'AHSTtol'

    zonenames = ([sub.replace(':', '_')
                  for sub
                  in ([zone.Name for zone in idf1.idfobjects['ZONE']])])
    for zonename in zonenames:
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:GlobalVariable']
                if x.Erl_Variable_1_Name == 'ComfHours_'+zonename])
        assert obj[0].Erl_Variable_1_Name == 'ComfHours_'+zonename
        assert obj[0].Erl_Variable_2_Name == 'DiscomfAppHotHours_'+zonename
        assert obj[0].Erl_Variable_3_Name == 'DiscomfAppColdHours_'+zonename
        assert obj[0].Erl_Variable_4_Name == 'DiscomfNonAppHotHours_'+zonename
        assert obj[0].Erl_Variable_5_Name == 'DiscomfNonAppColdHours_'+zonename
        assert obj[0].Erl_Variable_6_Name == 'ComfHoursNoApp_'+zonename

def test_addEMSSensorsSingleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addEMSSensorsSingleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    zonenames_orig = ([zone.Name for zone in idf1.idfobjects['ZONE']])
    zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in idf1.idfobjects['ZONE']])])

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Sensor']
            if x.Name == 'RMOT'])
    assert obj[0].Name == 'RMOT'
    assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == 'People ' + zonenames_orig[0]
    assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Thermal Comfort CEN 15251 Adaptive Model Running Average Outdoor Air Temperature'

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Sensor']
            if x.Name == 'PMOT'])
    assert obj[0].Name == 'PMOT'
    assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == 'People ' + zonenames_orig[0]
    assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Thermal Comfort ASHRAE 55 Adaptive Model Running Average Outdoor Air Temperature'

    for i in range(len(zonenames)):
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Sensor']
                if x.Name == zonenames[i] + '_OpT'])
        assert obj[0].Name == zonenames[i] + '_OpT'
        assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == zonenames_orig[i]
        assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Operative Temperature'

def test_addEMSActuatorsSingleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addEMSActuatorsSingleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Actuator']
            if x.Name == 'FORSCRIPT_AHST_Schedule'])
    assert obj[0].Name == 'FORSCRIPT_AHST_Schedule'
    assert obj[0].Actuated_Component_Unique_Name == 'FORSCRIPT_AHST'
    assert obj[0].Actuated_Component_Type == 'Schedule:Compact'
    assert obj[0].Actuated_Component_Control_Type == 'Schedule Value'

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Actuator']
            if x.Name == 'FORSCRIPT_ACST_Schedule'])
    assert obj[0].Name == 'FORSCRIPT_ACST_Schedule'
    assert obj[0].Actuated_Component_Unique_Name == 'FORSCRIPT_ACST'
    assert obj[0].Actuated_Component_Type == 'Schedule:Compact'
    assert obj[0].Actuated_Component_Control_Type == 'Schedule Value'

def test_addEMSProgramsSingleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addEMSProgramsSingleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')
    
    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Program']
            if x.Name == 'SetInputData'])
    assert obj[0].Name == 'SetInputData'
    assert obj[0].Program_Line_1 == 'set AdapStand = 1'
    assert obj[0].Program_Line_2 == 'set CAT = 1'
    assert obj[0].Program_Line_3 == 'set ComfMod = 1'
    assert obj[0].Program_Line_4 == 'set ACSTtol = -0.25'
    assert obj[0].Program_Line_5 == 'set AHSTtol = 0.25'

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Program']
            if x.Name == 'ApplyAST'])
    assert obj[0].Name == 'ApplyAST'
    assert obj[0].Program_Line_1 == 'Set FORSCRIPT_ACST_Schedule = ACST'
    assert obj[0].Program_Line_2 == 'Set FORSCRIPT_AHST_Schedule = AHST'

def test_addOutputVariablesSingleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addOutputVariablesSingleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    outputvariablelist = ([program.Name
                           for program
                           in idf1.idfobjects['EnergyManagementSystem:OutputVariable']])
    
    for outputvariable in outputvariablelist:
        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Variable_Name == outputvariable])
        assert obj[0].Key_Value == '*'
        assert obj[0].Variable_Name == outputvariable
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''
    
    addittionaloutputs = [
        'Zone Thermostat Operative Temperature',
        'Zone Thermal Comfort CEN 15251 Adaptive Model Running Average Outdoor Air Temperature',
        'Cooling Coil Total Cooling Rate',
        'Heating Coil Heating Rate',
        'Facility Total HVAC Electric Demand Power',
        'Facility Total HVAC Electricity Demand Rate',
        'VRF Heat Pump Cooling Electricity Energy',
        'VRF Heat Pump Heating Electricity Energy'
        ]
    
    for addittionaloutput in addittionaloutputs:
        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Variable_Name == addittionaloutput])
        assert obj[0].Key_Value == '*'
        assert obj[0].Variable_Name == addittionaloutput
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''

    obj = ([x
            for x
            in idf1.idfobjects['Output:Variable']
            if x.Variable_Name == 'Site Outdoor Air Drybulb Temperature'])
    assert obj[0].Key_Value == 'Environment'
    assert obj[0].Variable_Name == 'Site Outdoor Air Drybulb Temperature'
    assert obj[0].Reporting_Frequency == 'Hourly'
    assert obj[0].Schedule_Name == ''

    obj = ([x
            for x
            in idf1.idfobjects['Output:Variable']
            if x.Key_Value == 'FORSCRIPT_AHST'])
    assert obj[0].Key_Value == 'FORSCRIPT_AHST'
    assert obj[0].Variable_Name == 'Schedule Value'
    assert obj[0].Reporting_Frequency == 'Hourly'
    assert obj[0].Schedule_Name == ''

    obj = ([x
            for x
            in idf1.idfobjects['Output:Variable']
            if x.Key_Value == 'FORSCRIPT_ACST'])
    assert obj[0].Key_Value == 'FORSCRIPT_ACST'
    assert obj[0].Variable_Name == 'Schedule Value'
    assert obj[0].Reporting_Frequency == 'Hourly'
    assert obj[0].Schedule_Name == ''

    zonenames_orig = ([zone.Name for zone in idf1.idfobjects['ZONE']])
    for zonename in zonenames_orig:
        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Key_Value == zonename
                and x.Variable_Name == 'Zone Operative Temperature'])
        assert obj[0].Key_Value == zonename
        assert obj[0].Variable_Name == 'Zone Operative Temperature'
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''
